SystemOrganization addCategory: #'EjerciciosPoo-Measure'!

Exception subclass: #ConversionException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

TestCase subclass: #MeasureTest
	instanceVariableNames: 'distanceConverter timeConverter temperatureConverter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!MeasureTest methodsFor: 'errors' stamp: 'Paneyi 3/21/2012 12:30'!
canNotConvertMeasurementException
	
	^ConversionException! !

!MeasureTest methodsFor: 'temperature units' stamp: 'Paneyi 3/21/2012 12:23'!
celsius
	
	^SimpleUnit singularlyCalled: 'celsius' andPlurallyCalled: 'celsius' withConverter: temperatureConverter! !

!MeasureTest methodsFor: 'distance units' stamp: 'Paneyi 3/21/2012 12:22'!
centimeter
	
	^SimpleUnit singularlyCalled: 'centimeter' andPlurallyCalled: 'centimeters' withConverter: distanceConverter! !

!MeasureTest methodsFor: 'time units' stamp: 'Paneyi 3/21/2012 12:24'!
day
	
	^SimpleUnit singularlyCalled: 'day' andPlurallyCalled: 'days' withConverter: timeConverter! !

!MeasureTest methodsFor: 'distance units' stamp: 'Paneyi 3/21/2012 12:22'!
decameter
	
	^SimpleUnit singularlyCalled: 'decameter' andPlurallyCalled: 'decameters' withConverter: distanceConverter! !

!MeasureTest methodsFor: 'distance units' stamp: 'Paneyi 3/21/2012 12:22'!
decimeter
	
	^SimpleUnit singularlyCalled: 'decimeter' andPlurallyCalled: 'decimeters' withConverter: distanceConverter! !

!MeasureTest methodsFor: 'distance measures' stamp: 'HernanWilkinson 4/25/2011 17:27'!
distanceExamples 

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'distance units' stamp: 'HernanWilkinson 4/25/2011 17:27'!
distanceUnits 

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 12:20'!
eightCubicMeters

	| allUnits compoundUnit |
	
	allUnits := OrderedCollection with: self meter with: self meter with: self meter.
	compoundUnit := CompoundUnit withUnits: allUnits.
	
	^Measure withAmount: 8 in: compoundUnit.  ! !

!MeasureTest methodsFor: 'temperature units' stamp: 'Paneyi 3/21/2012 12:23'!
fahrenheit
	
	^SimpleUnit singularlyCalled: 'fahrenheit' andPlurallyCalled: 'fahrenheits' withConverter: temperatureConverter! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 15:23'!
finalSpeedStartingAt: initialSpeed accelerating: anAcceleration during: aTimePeriod

	| finalAmount allUnits finalUnit |
	
	finalAmount := anAcceleration amount * aTimePeriod amount + initialSpeed amount.
	
	allUnits := OrderedCollection new.
	initialSpeed unit units do: [ :eachUnit | allUnits add: eachUnit].
	finalUnit := CompoundUnit withUnits: allUnits.

	^Measure withAmount: finalAmount in: finalUnit! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 12:20'!
fourSquaredMeters

	| allUnits compoundUnit |
	
	allUnits := OrderedCollection with: self meter with: self meter.
	compoundUnit := CompoundUnit withUnits: allUnits.
	
	^Measure withAmount: 4 in: compoundUnit.  ! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/22/2012 11:13'!
generateDistanceConversionTableFromBase

	| table |
	
	table := ConversionTable new.
	
	table
		at: (self meter) store: [ :anAmount | anAmount * 1];
		at: (self decimeter) store: [ :anAmount | anAmount * 10 ];
		at: (self centimeter) store: [ :anAmount | anAmount * 100 ];
		at: (self millimeter) store: [ :anAmount | anAmount * 1000 ];
		at: (self decameter) store: [ :anAmount | anAmount * 10 reciprocal ];
		at: (self hectometer) store: [ :anAmount | anAmount * 100 reciprocal ];
		at: (self kilometer) store: [ :anAmount | anAmount * 1000 reciprocal ].
	
	^table! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/22/2012 11:13'!
generateDistanceConversionTableToBase

	| table |
	
	table := ConversionTable new.
	
	table
		at: (self meter) store: [ :anAmount | anAmount * 1 reciprocal ];
		at: (self decimeter) store: [ :anAmount | anAmount * 10 reciprocal ];
		at: (self centimeter) store: [ :anAmount | anAmount * 100 reciprocal ];
		at: (self millimeter) store: [ :anAmount | anAmount * 1000 reciprocal ];
		at: (self decameter) store: [ :anAmount | anAmount * 10 ];
		at: (self hectometer) store: [ :anAmount | anAmount * 100 ];
		at: (self kilometer) store: [ :anAmount | anAmount * 1000 ].
	
	^table! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/21/2012 09:59'!
generateDistanceConverterWith: fromBaseTable and: toBaseTable

	distanceConverter := Converter using: fromBaseTable and: toBaseTable withBase: self meter.! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/22/2012 11:13'!
generateTemperatureConversionTableFromBase

	| table |
	
	table := ConversionTable new.
	
	table
		at: (self kelvin) store: [ :anAmount | anAmount * 1 ];
		at: (self fahrenheit) store: [ :anAmount | (anAmount - 273.15) * 9 / 5 + 32 ];
		at: (self celsius) store: [ :anAmount | anAmount - 273.15 ].
	
	^table! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/22/2012 11:13'!
generateTemperatureConversionTableToBase

	| table |
	
	table := ConversionTable new.
	
	table
		at: (self kelvin) store: [ :anAmount | anAmount * 1 ];
		at: (self fahrenheit) store: [ :anAmount | (anAmount - 32) * 5 / 9 + 273.15 ];
		at: (self celsius) store: [ :anAmount | anAmount + 273.15 ].
	
	^table! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/21/2012 10:00'!
generateTemperatureConverterWith: fromBaseTable and: toBaseTable

	temperatureConverter := Converter using: fromBaseTable and: toBaseTable withBase: self kelvin.! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/22/2012 11:14'!
generateTimeConversionTableFromBase

	| table |
	
	table := ConversionTable new.
	
	table
		at: (self second) store: [ :anAmount | anAmount * 1];
		at: (self minute) store: [ :anAmount | anAmount * 60 reciprocal ];
		at: (self hour) store: [ :anAmount | anAmount * 3600 reciprocal ];
		at: (self day) store: [ :anAmount | anAmount * 86400 reciprocal ].
	
	^table! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/22/2012 11:14'!
generateTimeConversionTableToBase

	| table |
	
	table := ConversionTable new.
	
	table
		at: (self second) store: [ :anAmount | anAmount * 1 ];
		at: (self minute) store: [ :anAmount | anAmount * 60 ];
		at: (self hour) store: [ :anAmount | anAmount * 3600 ];
		at: (self day) store: [ :anAmount | anAmount * 86400 ].
	
	^table! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/21/2012 10:00'!
generateTimeConverterWith: fromBaseTable and: toBaseTable

	timeConverter := Converter using: fromBaseTable and: toBaseTable withBase: self second.! !

!MeasureTest methodsFor: 'distance units' stamp: 'Paneyi 3/21/2012 12:22'!
hectometer
	
	^SimpleUnit singularlyCalled: 'hectometer' andPlurallyCalled: 'hectometers' withConverter: distanceConverter! !

!MeasureTest methodsFor: 'time units' stamp: 'Paneyi 3/21/2012 12:24'!
hour
	
	^SimpleUnit singularlyCalled: 'hour' andPlurallyCalled: 'hours' withConverter: timeConverter! !

!MeasureTest methodsFor: 'temperature units' stamp: 'Paneyi 3/21/2012 12:23'!
kelvin
	
	^SimpleUnit singularlyCalled: 'kelvin' andPlurallyCalled: 'kelvin' withConverter: temperatureConverter! !

!MeasureTest methodsFor: 'distance units' stamp: 'Paneyi 3/21/2012 12:22'!
kilometer
	
	^SimpleUnit singularlyCalled: 'kilometer' andPlurallyCalled: 'kilometers' withConverter: distanceConverter! !

!MeasureTest methodsFor: 'objects in category' stamp: 'HernanWilkinson 4/25/2011 17:30'!
measureExamples 

	^ OrderedCollection new
		addAll: self distanceExamples;
		addAll: self temperatureExamples;
		addAll: self timeExamples;
		yourself ! !

!MeasureTest methodsFor: 'distance units' stamp: 'Paneyi 3/21/2012 12:23'!
meter

	^SimpleUnit singularlyCalled: 'meter' andPlurallyCalled: 'meters' withConverter: distanceConverter! !

!MeasureTest methodsFor: 'distance units' stamp: 'Paneyi 3/21/2012 12:22'!
millimeter
	
	^SimpleUnit singularlyCalled: 'milimeter' andPlurallyCalled: 'milimeters' withConverter: distanceConverter! !

!MeasureTest methodsFor: 'distance measures' stamp: 'Paneyi 3/21/2012 10:09'!
minusOneMeter 
	
	^Measure withAmount: -1 in: (self meter)! !

!MeasureTest methodsFor: 'time units' stamp: 'Paneyi 3/21/2012 12:24'!
minute
	
	^SimpleUnit singularlyCalled: 'minute' andPlurallyCalled: 'minutes' withConverter: timeConverter! !

!MeasureTest methodsFor: 'objects in category' stamp: 'HernanWilkinson 4/25/2011 17:27'!
objectsInCategoryOf: aSelector

	| category messageNames |
	
	category := self class organization categoryOfElement: aSelector.
	messageNames := (self class organization listAtCategoryNamed:  category) reject: [ :aMessageName | aMessageName = aSelector ].
	
	 ^ messageNames collect: [ :aMessageName | self perform: aMessageName ]
	

	
! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'Paneyi 3/21/2012 10:11'!
oneCelsius
	
	^Measure withAmount: 1 in: (self celsius)! !

!MeasureTest methodsFor: 'distance measures' stamp: 'Paneyi 3/21/2012 10:10'!
oneCentimeter
	
	^Measure withAmount: 1 in: (self centimeter)! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:12'!
oneDay
	
	^Measure withAmount: 1 in: (self day)! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:12'!
oneHour
	
	^Measure withAmount: 1 in: (self hour)! !

!MeasureTest methodsFor: 'distance measures' stamp: 'Paneyi 3/21/2012 10:10'!
oneHundredCentimeters
	
	^Measure withAmount: 100 in: (self centimeter)! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 12:20'!
oneHundredTwentyOneMetersPerSecond

	| allUnits compoundUnit |
	
	allUnits := OrderedCollection with: self meter with: self second reciprocal.
	compoundUnit := CompoundUnit withUnits: allUnits.
	
	^Measure withAmount: 121 in: compoundUnit.  ! !

!MeasureTest methodsFor: 'distance measures' stamp: 'Paneyi 3/21/2012 10:10'!
oneMeter
	
	^Measure withAmount: 1 in: (self meter)! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 12:20'!
oneMeterPerSecond

	| allUnits compoundUnit |
	
	allUnits := OrderedCollection with: self meter with: self second reciprocal.
	compoundUnit := CompoundUnit withUnits: allUnits.
	
	^Measure withAmount: 1 in: compoundUnit.  ! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:13'!
oneMinute
	
	^Measure withAmount: 1 in: (self minute)! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:13'!
oneSecond

	^Measure withAmount: 1 in: (self second)! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 12:20'!
oneSquaredMeter

	| allUnits compoundUnit |
	
	allUnits := OrderedCollection with: self meter with: self meter.
	compoundUnit := CompoundUnit withUnits: allUnits.
	
	^Measure withAmount: 1 in: compoundUnit.  ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 12:20'!
oneSquaredSecond

	| allUnits compoundUnit |
	
	allUnits := OrderedCollection with: self second with: self second.
	compoundUnit := CompoundUnit withUnits: allUnits.
	
	^Measure withAmount: 1 in: compoundUnit.  ! !

!MeasureTest methodsFor: 'time units' stamp: 'Paneyi 3/21/2012 12:24'!
second
	
	^SimpleUnit singularlyCalled: 'second' andPlurallyCalled: 'seconds' withConverter: timeConverter! !

!MeasureTest methodsFor: 'setUp' stamp: 'Paneyi 3/21/2012 11:54'!
setUp

	self generateDistanceConverterWith: self generateDistanceConversionTableFromBase and: self generateDistanceConversionTableToBase.
	
	self generateTimeConverterWith: self generateTimeConversionTableFromBase and: self generateTimeConversionTableToBase.
	
	self generateTemperatureConverterWith: self generateTemperatureConversionTableFromBase and: self generateTemperatureConversionTableToBase.! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:13'!
sixtyMinutes
	
	^Measure withAmount: 60 in: (self minute)! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:13'!
sixtyOneSeconds
	
	^Measure withAmount: 61 in: (self second)! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:13'!
sixtySeconds
	
	^Measure withAmount: 60 in: (self second)! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'HernanWilkinson 4/25/2011 17:28'!
temperatureExamples

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'temperature units' stamp: 'HernanWilkinson 4/25/2011 17:28'!
temperatureUnits

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:05'!
test01ObjectsRepresentingSameMeasureShouldBeEqual

	self assert: self oneMeter = self oneMeter.
	self assert: self oneSecond = self oneSecond.
	self assert: self oneCelsius = self oneCelsius.
	

! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:06'!
test02DifferentMeasuresShouldNotBeEqual

	self deny: self oneMeter = self twoMeters.
	self deny: self oneSecond = self sixtySeconds.
	self deny: self oneCelsius = self zeroCelsius.
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:07'!
test03SameMeasureExpressWithDifferentUnitsShouldBeEqual

	self assert: self oneMeter = self oneHundredCentimeters.
	self assert: self twoMeters = self twoHundredCentimeters.
	self assert: self oneMinute = self sixtySeconds.
	self assert: self zeroCelsius = self thirtyTwoFahrenheits.
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:08'!
test03aMeasuresOfDifferentDomainShouldNotBeEqual

	self deny: self oneMeter = self oneSecond.
	self deny: self oneMeter = self oneCelsius.
	self deny: self oneCelsius = self oneSecond
! !

!MeasureTest methodsFor: '1 - Test equality' stamp: 'HernanWilkinson 4/25/2011 16:09'!
test04EqualsShouldBeCommutative

	self assert: (self oneMeter = self oneHundredCentimeters and: [ self oneHundredCentimeters = self oneMeter ]).
	self assert: (self twoMeters = self twoHundredCentimeters and:  [ self twoHundredCentimeters = self twoMeters ]).
	self assert: (self oneMinute = self sixtySeconds and: [self sixtySeconds = self oneMinute]).
	self assert: (self zeroCelsius = self thirtyTwoFahrenheits and: [ self thirtyTwoFahrenheits = self zeroCelsius]).
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:11'!
test05UnitStringRepresentationShouldBeHumanReadable

	self assert: self meter printString = 'meter'.
	self assert: self centimeter printString = 'centimeter'.
	self assert: self decimeter printString = 'decimeter'.
	self assert: self millimeter printString = 'milimeter'.
	self assert: self decameter printString = 'decameter'.
	self assert: self hectometer printString = 'hectometer'.
	self assert: self kilometer printString = 'kilometer'.	
	
	self assert: self celsius printString = 'celsius'.	
	self assert: self fahrenheit printString = 'fahrenheit'.	

	self assert: self day printString = 'day'.	
	self assert: self hour printString = 'hour'.	
	self assert: self minute printString = 'minute'.	
	self assert: self second printString = 'second'.	

! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'Paneyi 3/21/2012 11:31'!
test06MeasureStringRepresentationShouldBeHumanReadable

	self assert: self oneMeter printString = '1 meter'.
	self assert: self twoMeters printString = '2 meters'.
	self assert: self oneHundredCentimeters printString = '100 centimeters'.
	
	self assert: self oneMinute printString = '1 minute'.
	self assert: self sixtySeconds printString = '60 seconds'.
	
	self assert: self zeroCelsius printString = '0 celsius'.
	self assert: self thirtyTwoFahrenheits printString = '32 fahrenheits'.
	
	
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:14'!
test07aMeasureAmountShouldBeTheExpectedAmount

	self assert: self oneMeter amount = 1.
	self assert: self twoMeters amount = 2.
	self assert: self oneHundredCentimeters amount = 100.
	self assert: self twoHundredCentimeters amount = 200.

	self assert: self oneMinute amount = 1.
	self assert: self sixtySeconds amount = 60.
	
	self assert: self zeroCelsius amount = 0.
	self assert: self thirtyTwoFahrenheits amount = 32.
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:15'!
test07bMeasureUnitShouldBeTheExpectedUnit

	self assert: self oneMeter unit = self meter.
	self assert: self twoMeters unit = self meter.
	self assert: self oneHundredCentimeters unit = self centimeter.
	self assert: self twoHundredCentimeters unit = self centimeter.
	
	self assert: self oneMinute unit = self minute.
	self assert: self sixtySeconds unit = self second.
	
	self assert: self zeroCelsius unit = self celsius.
	self assert: self thirtyTwoFahrenheits unit = self fahrenheit
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:15'!
test07cDifferentMeasureUnitsShouldBeDifferent

	| unitsGroupedByItself |
	
	unitsGroupedByItself := self units groupedBy: [ :aUnit | aUnit ].
	self assert: (unitsGroupedByItself allSatisfy: [:aCollectionOfUnit | aCollectionOfUnit size = 1] )
! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:03'!
test07dDistancesCanBeConvertedToDifferentUnits

	"A partir de los test de igualdad, se deben haber dado cuenta que debemos poder
	convertir distancias a distintas unidades... y que ademas dos distancias de distintas unidades
	pueden ser iguales... Por lo tanto por qu no hacer esta caracteristica de las distancias explicita?
	Veran que luego se torna muy util
	
	Respecto de como esta escrito el test, para no tener que escribir los assert para oneMeter y oneHundredCentimeters 
	creo un array y lo recorro... Se podria escribir menos aun en este test?
	
	Y por qu se asserta sobre el amount y luego la unidad y no directametne sobre la distancia completa. ej:
	self assert: (aDistance converTo: self meter) = self oneMeter
	"
	
	(Array with: self oneMeter with: self oneHundredCentimeters) do: [ :aDistance |
		self assert: (aDistance convertTo: self meter) amount = 1.
		self assert: (aDistance convertTo: self meter) unit = self meter.
		
		self assert: (aDistance convertTo: self decimeter) amount = 10.
		self assert: (aDistance convertTo: self decimeter) unit = self decimeter.
		
		self assert: (aDistance convertTo: self centimeter) amount = 100.
		self assert: (aDistance convertTo: self centimeter) unit = self centimeter.
		
		self assert: (aDistance convertTo: self millimeter) amount = 1000.
		self assert: (aDistance convertTo: self millimeter) unit = self millimeter.
	
		self assert: (aDistance convertTo: self decameter) amount = (1/10).
		self assert: (aDistance convertTo: self decameter) unit = self decameter.
		
		self assert: (aDistance convertTo: self hectometer) amount = (1/100).
		self assert: (aDistance convertTo: self hectometer) unit = self hectometer.
	
		self assert: (aDistance convertTo: self kilometer) amount = (1/1000).
		self assert: (aDistance convertTo: self kilometer) unit = self kilometer ]! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:17'!
test07eTimeMeasuresCanBeConvertedToDifferentUnits

	self assert: (self oneMinute convertTo: self second) = self sixtySeconds.
	self assert: (self sixtySeconds convertTo: self minute) = self oneMinute.

	self assert: (self oneHour convertTo: self minute) = self sixtyMinutes.
	self assert: (self sixtyMinutes convertTo: self hour) = self oneHour.

	self assert: (self oneDay convertTo: self hour) = self twentyFourHours.
	self assert: (self twentyFourHours convertTo: self day) = self oneDay.

! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:17'!
test07fTemperatureMeasuresCanBeConvertedToDifferentUnits

	self assert: (self zeroCelsius convertTo: self fahrenheit) = self thirtyTwoFahrenheits.
	self assert: (self thirtyTwoFahrenheits convertTo: self celsius) = self zeroCelsius.

! !

!MeasureTest methodsFor: '2 - Converting/accessing/printing' stamp: 'HernanWilkinson 4/25/2011 16:19'!
test07gMeasuresCanNotBeConvertedToUnitsOfDifferentDomain

	self should: [ self oneMeter convertTo: self second ] raise: self canNotConvertMeasurementException.
	self should: [ self oneSecond convertTo: self celsius ] raise: self canNotConvertMeasurementException.
	self should: [ self oneCelsius convertTo: self centimeter ] raise: self canNotConvertMeasurementException.


! !

!MeasureTest methodsFor: '3 - Test equality - implementation details' stamp: 'HernanWilkinson 4/25/2011 16:20'!
test08aComparingMeasuresWithOtherObjectsShouldReturnFalse

	"El igual no debe generar ningun tipo de error por mas que se comparen objetos
	de distintos tipos
	Fijense como se crea una instancia de Object sin ningun tipo de problema... qu representa?"

	self deny: self oneMeter = Object new.
	self deny: Object new = self oneMeter! !

!MeasureTest methodsFor: '3 - Test equality - implementation details' stamp: 'HernanWilkinson 4/25/2011 16:21'!
test08bMeasuresThatAreEqualShouldReturnTheSameHash

	"Debido a como funcionan los diccionarios, cuando dos objetos son iguales deben
	tener es mismo hash. La inversa no es verdad"
	
	self assert: self oneMeter hash=self oneMeter hash.
	self assert: self oneMeter hash=self oneHundredCentimeters hash.
	self assert: self oneMinute hash = self sixtySeconds hash.
	self assert: self zeroCelsius hash = self thirtyTwoFahrenheits hash.
	
	
! !

!MeasureTest methodsFor: '3 - Test equality - implementation details' stamp: 'HernanWilkinson 4/25/2011 16:21'!
test08cCopiesOfTheSameMeasureShouldBeEqual

	"Dos objetos que ocupan distintas posiciones de memoria pero representan
	la misma entidad de la realidad, deben devolver true cuando se los compara por igual, puesto
	que representan la misma entidad de la realidad.
	Por lo tanto, el mensaje #= significa 'representan el mismo ente de la realidad?'
	mientras que el mensaje #== significa ocupan la misma posicion de memoria?"

	| originalOneMeter copiedOneMeter |

	"Al objeto oneMeter lo copio para asegurarme de tener uno oneMeter en otra posicion de memoria"

	originalOneMeter := self oneMeter.
	copiedOneMeter := originalOneMeter copy.

	"Me aseguro que ocupen distinta posicion de memoria"
	self deny: originalOneMeter == copiedOneMeter.

	"Ahora me aseguro que sean iguales"
	self assert: originalOneMeter = copiedOneMeter! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:31'!
test09AddingMeasuresOfSameUnitShouldReturnAMeasureRepresentingTheAddingMeasures

	self assert: (self oneMeter + self oneMeter) = self twoMeters.
	self assert: (self oneSecond + self oneSecond) = self twoSeconds.
	self assert: (self oneCelsius + self oneCelsius) = self twoCelsius.
	

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:32'!
test10AddingMeasuresOfDifferentUnitShouldReturnAMeasureRepresentingTheAddingMeasures

	self assert: (self oneMeter + self oneHundredCentimeters) = self twoMeters.
	self assert: (self oneMeter + self oneHundredCentimeters) = self twoHundredCentimeters.
	self assert: (self oneSecond + self oneMinute) = self sixtyOneSeconds.

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:48'!
test10aCanNotAddMeasuresOfDifferentDomain

	self should: [ self oneMeter + self oneSecond ] raise: self canNotConvertMeasurementException 

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:32'!
test11AddingMeasuresIsCommutative

	self assert: (self oneMeter + self twoMeters)=(self twoMeters + self oneMeter).
	self assert: (self oneSecond + self twoSeconds)=(self twoSeconds + self oneSecond).
	self assert: (self oneCelsius + self twoCelsius)=(self twoCelsius + self oneCelsius)
	


! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:33'!
test12AddingMeasuresIsAssociative

	self assert: 
		((self oneMeter + self oneMeter) + self twoMeters) = 
		(self oneMeter + (self oneMeter + self twoMeters)).

	self assert: 
		((self oneSecond + self oneSecond) + self twoSeconds) = 
		(self oneSecond + (self oneSecond + self twoSeconds)).

	self assert: 
		((self oneCelsius + self oneCelsius) + self twoCelsius) = 
		(self oneCelsius + (self oneCelsius + self twoCelsius)).

		

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:35'!
test12aAddingMeasuresKeepsMeasureImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter + oneMeter.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond + oneSecond.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius + oneCelsius.
	
	self assert: oneCelsius = self oneCelsius 



! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:36'!
test13aNegatingAMeasureNegatesItsAmount

	(Array with: self oneMeter with: self twoSeconds with: self thirtyTwoFahrenheits) do: [:aMeasure |
		self assert: aMeasure negated amount=aMeasure amount negated ]
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:36'!
test13bNegatingAMeasureIsReflective

	(Array with: self oneMeter with: self twoSeconds with: self thirtyTwoFahrenheits) do: [:aMeasure |
		self assert: aMeasure negated negated=aMeasure ].
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:37'!
test14SubtractingMeasuresOfSameUnitShouldReturnAMeasureRepresentingTheSubtractedMeasures

	self assert: (self twoMeters - self oneMeter) = self oneMeter.
	self assert: (self oneMeter - self oneMeter) = self zeroMeters.
	self assert: (self oneMeter - self twoMeters) = self oneMeter negated.
	
	self assert: (self twoSeconds - self oneSecond) = self oneSecond.
	self assert: (self oneSecond - self oneSecond) = self zeroSeconds.
	self assert: (self oneSecond - self twoSeconds) = self oneSecond negated.
	
	self assert: (self twoCelsius - self oneCelsius) = self oneCelsius.
	self assert: (self oneCelsius - self oneCelsius) = self zeroCelsius.
	self assert: (self oneCelsius - self twoCelsius) = self oneCelsius negated.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:38'!
test15SubtractingMeasuresOfDifferentUnitShouldReturnAMeasureRepresentingTheSubtractedMeasures

	self assert: (self twoMeters - self oneHundredCentimeters) = self oneMeter.
	self assert: (self oneMeter - self oneHundredCentimeters) = self zeroMeters.
	self assert: (self oneHundredCentimeters - self twoMeters) = self oneMeter negated.
	
	self assert: (self sixtyOneSeconds - self oneMinute) = self oneSecond.
	

	! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:48'!
test15aCanNotSubtractMeasuresOfDifferentDomain

	self should: [ self oneMeter - self oneSecond ] raise: self canNotConvertMeasurementException 

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:38'!
test16SubtractingMeasuresIsAssociative

	self assert: 
		((self twoMeters - self oneMeter) - self oneMeter) = 
		(self twoMeters - (self oneMeter + self oneMeter)).

	self assert: 
		((self twoSeconds - self oneSecond) - self oneSecond) = 
		(self twoSeconds - (self oneSecond + self oneSecond)).	

	

	! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:39'!
test16aSubtractingMeasuresKeepsMeasuresImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter - oneMeter.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond - oneSecond.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius - oneCelsius.
	
	self assert: oneCelsius = self oneCelsius 


! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:40'!
test17aMeasuresCanBeMultiplyByAnInteger

	self assert: self oneMeter * 1 = self oneMeter.
	self assert: self oneMeter * 2 = self twoMeters.
	self assert: self oneMeter * 0 = self zeroMeters.

	self assert: self oneSecond * 1 = self oneSecond.
	self assert: self oneSecond * 2 = self twoSeconds.
	self assert: self oneSecond * 0 = self zeroSeconds.

	self assert: self oneCelsius * 1 = self oneCelsius.
	self assert: self oneCelsius * 2 = self twoCelsius.
	self assert: self oneCelsius * 0 = self zeroCelsius.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:40'!
test17bMeasuresCanBeMultiplyByAFraction

	self assert: (self oneMeter * (2/3)) amount = (2/3).
	self assert: (self twoMeters * (2/3)) amount = (2 * (2/3)).

	self assert: (self oneSecond * (2/3)) amount = (2/3).
	self assert: (self twoSeconds * (2/3)) amount = (2 * (2/3)).

	self assert: (self oneCelsius * (2/3)) amount = (2/3).
	self assert: (self twoCelsius * (2/3)) amount = (2 * (2/3)).
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:41'!
test17cMeasuresCanBeMultiplyByAFloat

	self assert: (self oneMeter * 2.5) amount = 2.5.
	self assert: (self twoMeters * 2.5) amount = (2 * 2.5).

	self assert: (self oneSecond * 2.5) amount = 2.5.
	self assert: (self twoSeconds * 2.5) amount = (2 * 2.5).

	self assert: (self oneCelsius * 2.5) amount = 2.5.
	self assert: (self twoCelsius * 2.5) amount = (2 * 2.5).
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:45'!
test17dMeasuresCanBeMultiplyByAMeasureOfSameDomain

	self assert: (self oneMeter * self oneMeter) = self oneSquaredMeter.
	self assert: (self twoMeters * self twoMeters) = self fourSquaredMeters.
	self assert: (self twoMeters * self twoMeters * self twoMeters) = self eightCubicMeters.

	self assert: (self oneSecond * self oneSecond) = self oneSquaredSecond.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:53'!
test17eMeasuresCanBeMultiplyByAMeasureOfDifferentDomain

	self assert: (self oneMeter * self twoSeconds) = self twoMetersTimesSecond.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:42'!
test18MultiplicationKeepsMeasureImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter * 2.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond * 2.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius * 2.
	
	self assert: oneCelsius = self oneCelsius 




! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:49'!
test19aMeasuresCanBeDividedByAnInteger

	self assert: (self oneMeter / 1) = self oneMeter.
	self assert: (self twoMeters / 2) = self oneMeter.

	self assert: (self oneSecond / 1) = self oneSecond.
	self assert: (self twoSeconds / 2) = self oneSecond.

	self assert: (self oneCelsius / 1) = self oneCelsius.
	self assert: (self twoCelsius / 2) = self oneCelsius.

! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:50'!
test19bMeasuresCanBeDividedByAFraction

	self assert: (self twoMeters / (1/3)) amount = 6.
	self assert: (self twoSeconds / (1/3)) amount = 6.
	self assert: (self twoCelsius / (1/3)) amount = 6.




! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:50'!
test19cMeasuresCanBeDividedByAFloat

	self assert: (self twoMeters / 2.5) amount = (2/2.5).
	self assert: (self twoSeconds / 2.5) amount = (2/2.5).
	self assert: (self twoCelsius / 2.5) amount = (2/2.5).




! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:52'!
test19dMeasuresCanBeDividedByAMeasureOfSameDomain

	self assert: (self oneMeter / self oneMeter) = 1.
	self assert: (self twoMeters / self oneMeter) = 2.
		
	self assert: (self oneSecond / self oneSecond) = 1.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'Paneyi 3/21/2012 15:11'!
test19eMeasuresCanBeDividedByAMeasureOfDifferentDomain

	self assert: (self twoMeters / self oneSecond) = self twoMetersBySecond.
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 16:54'!
test20DivisionKeepsMeasureImmutability

	| oneMeter oneSecond oneCelsius |
	
	oneMeter := self oneMeter.
	oneMeter / 2.
	
	self assert: oneMeter = self oneMeter.
	
	oneSecond := self oneSecond.
	oneSecond / 2.
	
	self assert: oneSecond = self oneSecond.
	
	oneCelsius := self oneCelsius.
	oneCelsius / 2.
	
	self assert: oneCelsius = self oneCelsius 


! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'efo 5/2/2011 20:56'!
test20aFinalSpeedCanBeRepresentedWithMeasures

	self assert: 
		(self finalSpeedStartingAt: self oneMeterPerSecond accelerating: self twoMetersPerSquaredSecond during: self sixtySeconds) = 
		(self oneMeterPerSecond + (self twoMetersPerSquaredSecond * self sixtySeconds)).
		
	self assert: (self finalSpeedStartingAt: self oneMeterPerSecond accelerating: self twoMetersPerSquaredSecond during: self sixtySeconds) = self oneHundredTwentyOneMetersPerSecond 
	
! !

!MeasureTest methodsFor: '4 - Test arithmetic operations' stamp: 'HernanWilkinson 4/25/2011 17:00'!
test20bMultiplyingANumberByAUnitReturnsAMeasure

	self assert: (1*self meter) = self oneMeter.
	self assert: (2*self second) = self twoSeconds.
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:02'!
test21MeasuresProvideATotalOrder

	self assert: (self measureExamples allSatisfy: [ :aMeasure | aMeasure < (aMeasure + (1* aMeasure unit))])

! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:03'!
test22MeasuresCanBeComparedWithGreaterThan

	self assert: (self measureExamples allSatisfy: [ :aMeasure | (aMeasure + (1 * aMeasure unit)) > aMeasure ])
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:04'!
test23MeasuresCanBeComparedWithLessThanOrEqual

	self assert: (self measureExamples allSatisfy: [ :aMeasure | aMeasure <= aMeasure and: [aMeasure <= (aMeasure + (1 * aMeasure unit))]])

	
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test24MeasuresCanBeComparedWithGreaterThanOrEqual

	self assert: (self measureExamples allSatisfy: [ :aMeasure | aMeasure >= aMeasure and: [(aMeasure + (1 * aMeasure unit)) >= aMeasure]])! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test25MaxReturnsTheMaximumBetweenTwoMeasures

	self assert: (self twoMeters max: self oneMeter) = self twoMeters.
	self assert: (self oneMeter max: self twoMeters) = self twoMeters.

	self assert: (self twoMeters max: self oneHundredCentimeters) = self twoMeters.
	self assert: (self oneHundredCentimeters max: self twoMeters) = self twoMeters.

	
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test26MinReturnsTheMinimumBetweenTwoMeasures

	self assert: (self twoMeters min: self oneMeter) = self oneMeter.
	self assert: (self oneMeter min: self twoMeters) = self oneMeter.

	self assert: (self twoMeters min: self oneHundredCentimeters) = self oneHundredCentimeters.
	self assert: (self oneHundredCentimeters min: self twoMeters) = self oneHundredCentimeters.

	
! !

!MeasureTest methodsFor: '5 - Test magnitude protocol' stamp: 'HernanWilkinson 4/25/2011 17:05'!
test27BetweenAndReturnsTrueIfAMeasureIsBetweenOtherTwoMeasures

	self assert: (self oneMeter between: self zeroMeters and: self twoMeters).
	self assert: (self oneHundredCentimeters between: self zeroMeters and: self twoMeters).
	
	self deny: (self twoMeters between: self zeroMeters and: self oneMeter).
	self deny: (self twoHundredCentimeters between: self zeroMeters and: self oneMeter).

! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'Paneyi 3/22/2012 09:29'!
test28IntervalOfMeasuresCanBeCreatedBecauseTheyDefineATotalOrder

	self assert: (self oneMeter to: self twoMeters) first=self oneMeter.
	self assert: (self oneMeter to: self twoMeters) last=self twoMeters.
	self assert: (self oneMeter to: self twoMeters) increment =self oneMeter.	

	self assert: (self oneHundredCentimeters to: self twoMeters) first=self oneMeter.
	self assert: (self oneHundredCentimeters to: self twoMeters) last=self twoMeters.
	self assert: (self oneHundredCentimeters to: self twoMeters) increment=self oneHundredCentimeters.	
	! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:16'!
test29MeasureIntervalStepCanBeAnyMeasure

	self assert: (self zeroMeters to: (self twoMeters * 2) by: self twoMeters) first=self zeroMeters.
	self assert: (self zeroMeters to: (self twoMeters * 2) by: self twoMeters) last=(self twoMeters * 2).
	self assert: (self zeroMeters to: (self twoMeters * 2) by: self twoMeters) increment=self twoMeters.

	self assert: ((self twoMeters * 2) to: self zeroMeters by: self twoMeters negated) first=(self twoMeters * 2).
	self assert: ((self twoMeters * 2) to: self zeroMeters by: self twoMeters negated) last=self zeroMeters.
	self assert: ((self twoMeters * 2) to: self zeroMeters by: self twoMeters negated) increment=self twoMeters negated

	! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:17'!
test30MeasureIntervalSizeReturnsTheNumberOfMeasuresOfTheInterval

	self assert: (self zeroMeters to: (self twoMeters * 2) ) size =5.
	self assert: ((self twoMeters * 2) to: self zeroMeters by: self oneMeter negated) size =5.
! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:17'!
test31MeasureIntervalCanBeEnumeratedUsingDo

	| value |
	
	value := 1.
	(self oneMeter to: (self oneMeter * 10)) do: [ :aMeasure |
		self assert: aMeasure amount = value.
		value := value + 1 ].
	
	self assert: value = 11
! !

!MeasureTest methodsFor: '6 - Test interval protocol' stamp: 'HernanWilkinson 4/25/2011 17:18'!
test32MeasureIntervalCanBeEnumeratedWithSpecificStep

	| value initialDistance byDistance finalDistance |
	
	initialDistance := self oneMeter.
	byDistance := self twoMeters.
	finalDistance := initialDistance * 9.
	value := initialDistance amount.
		
	(initialDistance to: finalDistance by:  byDistance) do: [ :aMeasure |
		self assert: aMeasure amount = value.
		value := value + byDistance amount ].
	
	self assert:  value = (finalDistance amount + byDistance amount)
! !

!MeasureTest methodsFor: '7 - Polymorphic zero' stamp: 'HernanWilkinson 4/25/2011 17:20'!
test33NothingOnAnyDomainAreEquals

	self assert: self zeroMeters = self zeroSeconds.
	self assert: self zeroSeconds = self zeroKelvin.
	self assert: self zeroKelvin = self zeroMeters! !

!MeasureTest methodsFor: '7 - Polymorphic zero' stamp: 'HernanWilkinson 4/25/2011 17:42'!
test34ZeroCelsiusIsNotNothingOnTemperature

	self deny: self zeroCelsius = self zeroKelvin.
	self deny: self zeroSeconds = self zeroCelsius.
! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'Paneyi 3/21/2012 10:11'!
thirtyTwoFahrenheits
	
	^Measure withAmount: 32 in: (self fahrenheit)! !

!MeasureTest methodsFor: 'time measures' stamp: 'HernanWilkinson 4/25/2011 17:29'!
timeExamples

	^self objectsInCategoryOf: thisContext method selector.
	
! !

!MeasureTest methodsFor: 'time units' stamp: 'HernanWilkinson 4/29/2011 17:10'!
timeUnits

	^self objectsInCategoryOf: thisContext method selector.
	
 ! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:13'!
twentyFourHours
	
	^Measure withAmount: 24 in: (self hour)! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'Paneyi 3/21/2012 10:11'!
twoCelsius
	
	^Measure withAmount: 2 in: (self celsius)! !

!MeasureTest methodsFor: 'distance measures' stamp: 'Paneyi 3/21/2012 10:10'!
twoHundredCentimeters
	
	^Measure withAmount: 200 in: (self centimeter)! !

!MeasureTest methodsFor: 'distance measures' stamp: 'Paneyi 3/21/2012 10:10'!
twoMeters
	
	^Measure withAmount: 2 in: (self meter)! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 12:20'!
twoMetersBySecond

	| allUnits compoundUnit |
	
	allUnits := OrderedCollection with: self meter with: self second reciprocal.
	compoundUnit := CompoundUnit withUnits: allUnits.
	
	^Measure withAmount: 2 in: compoundUnit.  ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 12:20'!
twoMetersPerSquaredSecond

	| allUnits compoundUnit |
	
	allUnits := OrderedCollection with: self meter with: self second reciprocal with: self second reciprocal.
	compoundUnit := CompoundUnit withUnits: allUnits.
	
	^Measure withAmount: 2 in: compoundUnit.  ! !

!MeasureTest methodsFor: 'compound measures' stamp: 'Paneyi 3/22/2012 12:20'!
twoMetersTimesSecond

	| allUnits compoundUnit |
	
	allUnits := OrderedCollection with: self meter with: self second.
	compoundUnit := CompoundUnit withUnits: allUnits.
	
	^Measure withAmount: 2 in: compoundUnit.  ! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:13'!
twoSeconds
	
	^Measure withAmount: 2 in: (self second)! !

!MeasureTest methodsFor: 'objects in category' stamp: 'HernanWilkinson 4/25/2011 17:34'!
units 

	^ OrderedCollection new
		addAll: self distanceUnits;
		addAll: self temperatureUnits;
		addAll: self timeUnits;
		yourself ! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'Paneyi 3/21/2012 10:11'!
zeroCelsius
	
	^Measure withAmount: 0 in: (self celsius)! !

!MeasureTest methodsFor: 'temperature measures' stamp: 'Paneyi 3/21/2012 10:11'!
zeroKelvin
	
	^Measure withAmount: 0 in: (self kelvin)! !

!MeasureTest methodsFor: 'distance measures' stamp: 'Paneyi 3/21/2012 10:11'!
zeroMeters

	^Measure withAmount: 0 in: (self meter)! !

!MeasureTest methodsFor: 'time measures' stamp: 'Paneyi 3/21/2012 10:13'!
zeroSeconds
	
	^Measure withAmount: 0 in: (self second)! !

Object subclass: #ConversionTable
	instanceVariableNames: 'conversionMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!ConversionTable methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 11:13'!
at: anIndex store: aValue

	conversionMap at: anIndex put: aValue! !

!ConversionTable methodsFor: 'initialization' stamp: 'Paneyi 3/21/2012 09:45'!
initialize

	conversionMap := Dictionary new.! !

!ConversionTable methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 11:11'!
retrieveValueAt: anIndex

	^conversionMap at: anIndex! !

Object subclass: #Converter
	instanceVariableNames: 'conversionTableFromBase conversionTableToBase baseUnit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!Converter class methodsFor: 'as yet unclassified' stamp: 'Paneyi 3/21/2012 09:57'!
using: fromBaseTable and: toBaseTable withBase: aUnit

	^self new initializeUsing: fromBaseTable and: toBaseTable withBase: aUnit! !

!Converter methodsFor: 'accessing' stamp: 'Paneyi 3/21/2012 10:54'!
baseUnit

	^baseUnit copy! !

!Converter methodsFor: 'accessing' stamp: 'Paneyi 3/22/2012 11:15'!
baseUnitFor: aUnit

	^baseUnit copy incrementExponentBy: (aUnit exponent -1)! !

!Converter methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 12:09'!
convert: aMeasure to: targetUnit

	| baseMeasure convertedMeasure |
	
	baseMeasure := self convertToBase: aMeasure.
	convertedMeasure := self convertFromBase: baseMeasure to: targetUnit.
	
	^ convertedMeasure! !

!Converter methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 15:29'!
convertAmountFromBase: baseAmount to: targetUnit

	| convertedAmount |
	
	convertedAmount := baseAmount.
	targetUnit exponent abs
		timesRepeat: [ convertedAmount := (conversionTableFromBase retrieveValueAt: targetUnit flatUnit) value: convertedAmount ].
		
	^ convertedAmount! !

!Converter methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 15:30'!
convertAmountToBase: anAmount from: aUnit

	| convertedAmount |
	
	convertedAmount := anAmount.
	
	aUnit exponent abs
		timesRepeat: [ convertedAmount := (conversionTableToBase retrieveValueAt: aUnit flatUnit) value: convertedAmount ].
		
	^ convertedAmount! !

!Converter methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 12:19'!
convertFromBase: baseMeasure to: targetUnit

	| convertedAmount |
	
	convertedAmount := self convertAmountFromBase: baseMeasure amount to: targetUnit.
	
	^Measure withAmount: convertedAmount in: targetUnit. ! !

!Converter methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 12:18'!
convertToBase: aMeasure

	| baseAmount |
	
	baseAmount := self convertAmountToBase: aMeasure amount from: aMeasure unit.
	
	^Measure withAmount: baseAmount in: (self baseUnitFor: aMeasure unit). ! !

!Converter methodsFor: 'initialization' stamp: 'Paneyi 6/5/2012 16:20'!
initializeUsing: fromBaseTable and: toBaseTable withBase: aUnit

	"El setConverter: es necesario por la forma de inicializar, ya que el conversor necesita la unidad base
	y esta necesita del conversor; en esta referencia cclica, la unidad pasada como parmetro termina con un
	conversor nil"

	conversionTableFromBase := fromBaseTable.
	conversionTableToBase := toBaseTable.
	aUnit setConverter: self.
	baseUnit := aUnit! !

Object subclass: #Measure
	instanceVariableNames: 'amount unit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!Measure class methodsFor: 'as yet unclassified' stamp: 'Paneyi 3/21/2012 10:09'!
withAmount: anAmount in: aUnit

	^super new initializeWithAmount: anAmount in: aUnit.! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 12:49'!
* anObject
	
	^anObject multiplyByMeasure: self! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 14:36'!
+ anotherMeasure

	^Measure withAmount: ((self amount) + (anotherMeasure convertTo: self unit) amount) in: self unit! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 12:32'!
- anotherMeasure

	^Measure withAmount: ((self amount) - (anotherMeasure convertTo: self unit) amount) in: self unit! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 12:30'!
/ anObject
	
	^self * anObject reciprocal! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 12:27'!
< anotherMeasure

	^ self amount < (anotherMeasure convertTo: self unit) amount! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 12:27'!
<= anotherMeasure

	^ self amount <= (anotherMeasure convertTo: self unit) amount! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 11:06'!
= anObject

	"^(self class = anObject class) and: [self convertAmountToBase = anObject convertAmountToBase] and: [self convertUnitToBase = anObject convertUnitToBase]"
	
	^ (self convertAmountToBase = 0 and: [ anObject convertAmountToBase = 0 ])
		or: [(self class = anObject class)
			and: [self convertAmountToBase = anObject convertAmountToBase]
			and: [self convertUnitToBase = anObject convertUnitToBase]]! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 12:27'!
> anotherMeasure

	^ self amount > (anotherMeasure convertTo: self unit) amount! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 12:27'!
>= anotherMeasure

	^ self amount >= (anotherMeasure convertTo: self unit) amount! !

!Measure methodsFor: 'accessing' stamp: 'Paneyi 3/21/2012 10:19'!
amount

	^amount! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 12:28'!
between: lowerMeasure and: higherMeasure

	^ self amount between: (lowerMeasure convertTo: self unit) amount and: (higherMeasure convertTo: self unit) amount! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 10:21'!
convertAmountToBase

	^self unit convertAmountToBase: self amount! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 12:29'!
convertTo: aUnit
	
	^ (self sameDomainAs: aUnit)
		ifTrue: [ self unit convert: self to: aUnit ]
		ifFalse: [ ConversionException signal ]! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 10:53'!
convertUnitToBase

	^self unit convertUnitToBase! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 12:29'!
hash

	^self convertAmountToBase hash bitXor: self convertUnitToBase hash! !

!Measure methodsFor: 'initialization' stamp: 'Paneyi 3/21/2012 10:09'!
initializeWithAmount: anAmount in: aUnit.

	amount := anAmount.
	unit := aUnit! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 12:29'!
max: anotherMeasure

	^ self < (anotherMeasure convertTo: self unit)
		ifTrue: [ anotherMeasure ]
		ifFalse: [ self ]! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 12:29'!
min: anotherMeasure

	^ self > (anotherMeasure convertTo: self unit)
		ifTrue: [ anotherMeasure ]
		ifFalse: [ self ]! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 11:03'!
multiplyByMeasure: aMeasure

	| resultingAmount resultingUnit resultingMeasure |
	
	resultingAmount := self amount * aMeasure amount.
	resultingUnit := self unit * aMeasure unit.
	
	^(resultingUnit hasNoUnits)
		ifTrue: [resultingAmount]
		ifFalse: [resultingMeasure := Measure withAmount: resultingAmount in: resultingUnit.]
	
	
	
	
	
! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 12:51'!
multiplyByNumber: aNumber
	
	^Measure withAmount: (self amount * aNumber) in: self unit! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 12:35'!
negated

	^ Measure withAmount: self amount negated in: self unit! !

!Measure methodsFor: 'printing' stamp: 'Paneyi 3/21/2012 11:30'!
printOn: aStream

	aStream
		nextPutAll: self amount printString;
		space.

	self amount ~= 1
		ifTrue: [ aStream nextPutAll: self unit pluralName ]
		ifFalse: [ aStream nextPutAll: self unit singularName ].
		
	^ aStream! !

!Measure methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 09:19'!
reciprocal

	self unit reciprocal ! !

!Measure methodsFor: 'comparing' stamp: 'Paneyi 3/21/2012 12:30'!
sameDomainAs: aUnit
	
	^ self unit convertUnitToBase = aUnit convertUnitToBase! !

!Measure methodsFor: 'instance creation' stamp: 'Paneyi 3/22/2012 09:28'!
to: anotherDistance

	^ MeasureInterval
		from: self
		to: anotherDistance
		by: (Measure withAmount: 1 in: self unit convertUnitToBase)! !

!Measure methodsFor: 'instance creation' stamp: 'Paneyi 3/22/2012 09:27'!
to:finalDistance by: intervalDistance

	^MeasureInterval from: self to: finalDistance by: intervalDistance
! !

!Measure methodsFor: 'accessing' stamp: 'Paneyi 3/21/2012 10:19'!
unit

	^unit! !

Object subclass: #MeasureInterval
	instanceVariableNames: 'step fromMeasure toMeasure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!MeasureInterval class methodsFor: 'as yet unclassified' stamp: 'Paneyi 3/19/2012 15:31'!
from: initialMeasure to: finalMeasure by: intervalStep 

	^super new initializeFrom: initialMeasure to: finalMeasure by: intervalStep.! !

!MeasureInterval methodsFor: 'operation' stamp: 'Paneyi 3/19/2012 15:26'!
do: aBlock
																
	^(self first amount to: self last amount by: self increment amount) do: [ :counter | aBlock value: (DistanceMeasure withLength: counter in: self first unit)]! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Paneyi 3/19/2012 15:32'!
first

	^fromMeasure! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Paneyi 3/19/2012 15:23'!
increment

	^step! !

!MeasureInterval methodsFor: 'initialization' stamp: 'Paneyi 3/19/2012 15:32'!
initializeFrom: initialMeasure to: lastMeasure by: intervalStep

	fromMeasure := initialMeasure convertTo: (intervalStep unit).
	toMeasure := lastMeasure convertTo: (intervalStep unit).
	step := intervalStep! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Paneyi 3/19/2012 15:32'!
last

	^toMeasure! !

!MeasureInterval methodsFor: 'accessing' stamp: 'Paneyi 3/19/2012 15:26'!
size

	^((((self first max: self last) - (self first min: self last)) / self increment amount) amount abs) +1! !

Object subclass: #Unit
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

Unit subclass: #CompoundUnit
	instanceVariableNames: 'units'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!CompoundUnit class methodsFor: 'as yet unclassified' stamp: 'Paneyi 3/21/2012 13:15'!
withUnits: aBagOfUnits

	^super new initializeWithUnits: aBagOfUnits! !

!CompoundUnit methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 15:47'!
* aUnit

	^aUnit multiplyByCompundUnit: self! !

!CompoundUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 10:50'!
/ aUnit

	^self * aUnit reciprocal! !

!CompoundUnit methodsFor: 'comparing' stamp: 'Paneyi 3/21/2012 12:39'!
= anObject

	^(self class = anObject class) and: [self units = anObject units]! !

!CompoundUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 11:26'!
addToUnits: aUnit
	
	| result |
	
	result := units detect: [ :unit | unit sameDomainAs: aUnit ] ifNone: [nil].
	^ result isNil
		ifTrue: [ units add: aUnit ]
		ifFalse: [ result incrementExponentBy: aUnit exponent ]
		
	"units
		detect: [ :unit | unit convertUnitToBase = aUnit convertUnitToBase ] 
		ifNone: [ units add: aUnit ]	
		ifOne: [ :unit | unit incrementExponentBy: aUnit exponent ]"! !

!CompoundUnit methodsFor: 'converting' stamp: 'Paneyi 3/22/2012 15:55'!
convert: aMeasure to: aCompoundUnit

	| convertedAmount toUnit |
	
	convertedAmount := aMeasure amount.
	
	units do: [ :fromUnit | 
		toUnit := aCompoundUnit units detect: [ :eachToUnit | fromUnit sameDomainAs: eachToUnit ].
		convertedAmount := fromUnit convertAmount: convertedAmount to: toUnit].
	
	^Measure withAmount: convertedAmount in: aCompoundUnit.

	"^aMeasure"! !

!CompoundUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 15:04'!
convertAmountToBase: anAmount

	^self units inject: anAmount into: [ :tempAmount :eachUnit | eachUnit convertAmountToBase: tempAmount]
	
	! !

!CompoundUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 12:21'!
convertUnitToBase

	| newBaseUnit |
	
	newBaseUnit := OrderedCollection new.
	
	self units do: [ :eachUnit | newBaseUnit add: eachUnit convertUnitToBase].
	
	^CompoundUnit withUnits: newBaseUnit! !

!CompoundUnit methodsFor: 'testing' stamp: 'Paneyi 3/22/2012 11:04'!
hasNoUnits 

	^units isEmpty! !

!CompoundUnit methodsFor: 'initialization' stamp: 'Paneyi 3/22/2012 15:42'!
initializeWithUnits: aCollectionOfUnits

	units := Set new.
	aCollectionOfUnits do: [ :unit | self addToUnits: unit ].
	units := units reject: [ :unit | unit exponent = 0]. ! !

!CompoundUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 12:36'!
multiplyByCompundUnit: aUnit

	| allUnits resultingUnit |
	
	allUnits := OrderedCollection new.
	aUnit units do: [ :eachUnit | allUnits add: eachUnit].
	self units do: [ :eachUnit | allUnits add: eachUnit].
	
	^CompoundUnit withUnits: allUnits! !

!CompoundUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 12:36'!
multiplyBySimpleUnit: aUnit

	| allUnits resultingUnit |
	
	allUnits := OrderedCollection with: aUnit.
	self units do: [ :eachUnit | allUnits add: eachUnit].
	
	^CompoundUnit withUnits: allUnits! !

!CompoundUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 12:35'!
reciprocal

	| newReciprocalUnit |
	
	newReciprocalUnit := OrderedCollection new.
	
	self units do: [ :eachUnit | newReciprocalUnit add: eachUnit reciprocal ].
	
	^CompoundUnit withUnits: newReciprocalUnit! !

!CompoundUnit methodsFor: 'accessing' stamp: 'Paneyi 3/21/2012 12:40'!
units

	^units! !

Unit subclass: #SimpleUnit
	instanceVariableNames: 'singularName pluralName exponent converter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-Measure'!

!SimpleUnit class methodsFor: 'as yet unclassified' stamp: 'Paneyi 3/22/2012 11:51'!
singularlyCalled: singularUnitName andPlurallyCalled: pluralUnitName withConverter: aConverter

	^self new initializeSingularlyCalled: singularUnitName andPlurallyCalled: pluralUnitName withConverter: aConverter! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 15:21'!
* aUnit

	^aUnit multiplyBySimpleUnit: self.! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 10:50'!
/ aUnit

	^self reciprocal * aUnit! !

!SimpleUnit methodsFor: 'comparing' stamp: 'Paneyi 3/21/2012 11:17'!
= anObject

	^(self class = anObject class) and: [self singularName = anObject singularName] and: [self exponent = anObject exponent]! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 11:34'!
convert: aMeasure to: aUnit

	^converter convert: aMeasure to: aUnit! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 15:58'!
convertAmount: anAmount to: aUnit
	
	| originalMeasure convertedMeasure |
	
	originalMeasure := Measure withAmount: anAmount in: self.
	
	convertedMeasure := converter convert: originalMeasure to: aUnit.
	
	^convertedMeasure amount.! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 12:27'!
convertAmountToBase: anAmount

	^converter convertAmountToBase: anAmount from: self! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 12:27'!
convertUnitToBase

	^converter baseUnitFor: self! !

!SimpleUnit methodsFor: 'accessing' stamp: 'Paneyi 3/21/2012 10:43'!
converter

	^converter! !

!SimpleUnit methodsFor: 'accessing' stamp: 'Paneyi 3/21/2012 10:18'!
exponent

	^exponent! !

!SimpleUnit methodsFor: 'accessing' stamp: 'Paneyi 3/22/2012 10:47'!
flatUnit

	^self copy incrementExponentBy: (exponent negated + 1)! !

!SimpleUnit methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 12:15'!
hash

	^singularName hash bitXor: exponent hash ! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 10:04'!
incrementExponentBy: anExponent

	exponent := exponent + anExponent! !

!SimpleUnit methodsFor: 'initialization' stamp: 'Paneyi 3/21/2012 12:25'!
initializeSingularlyCalled: singularUnitName andPlurallyCalled: pluralUnitName withConverter: aConverter

	singularName := singularUnitName.
	pluralName := pluralUnitName.
	exponent := 1.
	converter := aConverter! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 12:21'!
multiplyByCompundUnit: aUnit

	| allUnits resultingUnit |
	
	allUnits := OrderedCollection with: self.
	aUnit units do: [ :eachUnit | allUnits add: eachUnit].
	resultingUnit := CompoundUnit withUnits: allUnits.
	
	^resultingUnit! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/22/2012 12:21'!
multiplyBySimpleUnit: aUnit

	| allUnits resultingUnit |
	
	allUnits := OrderedCollection with: self with: aUnit.
	resultingUnit := CompoundUnit withUnits: allUnits.
	
	^resultingUnit! !

!SimpleUnit methodsFor: 'accessing' stamp: 'Paneyi 3/21/2012 10:18'!
pluralName

	^pluralName! !

!SimpleUnit methodsFor: 'printing' stamp: 'Paneyi 3/21/2012 11:30'!
printOn: aStream

	aStream nextPutAll: singularName! !

!SimpleUnit methodsFor: 'operation' stamp: 'Paneyi 3/21/2012 15:04'!
reciprocal

	exponent := exponent negated! !

!SimpleUnit methodsFor: 'comparing' stamp: 'Paneyi 3/22/2012 11:27'!
sameDomainAs: aUnit
	
	^ self flatUnit = aUnit flatUnit! !

!SimpleUnit methodsFor: 'initialization' stamp: 'Paneyi 3/22/2012 09:24'!
setConverter: aConverter

	converter := aConverter! !

!SimpleUnit methodsFor: 'accessing' stamp: 'Paneyi 3/21/2012 10:18'!
singularName

	^singularName! !

!Unit class methodsFor: 'as yet unclassified' stamp: 'Jamardo 5/22/2012 23:54'!
named: aName withFactor: aFactor

	^self new initializeNamed: aName whitFactor: aFactor! !

!Unit methodsFor: 'unitOperations' stamp: 'Jamardo 5/22/2012 23:16'!
= anUnit

	^name = anUnit name and:[factor = anUnit factor]! !

!Unit methodsFor: 'converting' stamp: 'Paneyi 3/21/2012 12:56'!
adaptToInteger: rcvr andSend: selector
	
	^Measure withAmount: rcvr in: self! !

!Unit methodsFor: 'accessors' stamp: 'Jamardo 5/22/2012 22:37'!
factor

	^factor! !

!Unit methodsFor: 'unitOperations' stamp: 'Jamardo 5/22/2012 22:18'!
initializeNamed: aName whitFactor: aFactor

	name := aName.
	factor := aFactor
	! !

!Unit methodsFor: 'accessors' stamp: 'Jamardo 5/22/2012 23:14'!
name

	^name! !

!Unit methodsFor: 'unitOperations' stamp: 'Jamardo 5/23/2012 00:22'!
offsetTo: anUnit

	^factor / anUnit factor! !

!Unit methodsFor: 'unitOperations' stamp: 'Jamardo 5/22/2012 22:53'!
printOn: aStream

	aStream nextPutAll: name! !
